/* -*- Mode: C -*-
 * This file is part Tempura: Interval Temporal Logic interpreter.
 *  
 * Copyright (C) 1987-2013  Roger Hale, Ben Moszkowski, Antonio Cau 
 *
 * Tempura is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Tempura is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Tempura.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/* 
 * Parallel summing of the nodes of a binary tree.
 * Ben Moszkowski 
 */

load "library".

define sum_tree(Tree) = {
    if is_integer(Tree) then empty
    else {
	exists Done:{
	    (Done is (is_integer(Tree[0]) and is_integer(Tree[1])))
	    and halt(Done)
	    and stable(struct(Tree))
	    and sum_tree_process(Done,Tree[0])
	    and sum_tree_process(Done,Tree[1])
	};
	{
	    skip and Tree:=Tree[0]+Tree[1]
	}
    }
}.

/* Simple process to sum a tree */
define sum_tree_process(Done,Tree) = process {
    sum_tree(Tree);
    {
	halt Done
	and stable Tree
    }
}.

define sum_test(t) = {
    exists Tree : {
	Tree=t
	and sum_tree(Tree)
        and always format("Tree=%t\n",Tree)
    }
}.

define sum_test1 = 1.

define sum_test2 = [1,2].

define sum_test3 = [[1,[2,3]],[4,5]].

define sum_test4 = [[[1,1],[1,1]],[[1,1],[1,1]]].

define sum_test5 = [[sum_test3,sum_test3],sum_test3].

/* run */ define run_test() = {
 {format("******* test 1 *******\n") and sum_test(sum_test1)};
 {format("******* test 2 *******\n") and sum_test(sum_test2)};
 {format("******* test 3 *******\n") and sum_test(sum_test3)};
 {format("******* test 4 *******\n") and sum_test(sum_test4)};
 {format("******* test 5 *******\n") and sum_test(sum_test5)}
}.

