/* -*- Mode: C -*-
 * This file is part Tempura: Interval Temporal Logic interpreter.
 *  
 * Copyright (C) 1987-2013  Roger Hale, Ben Moszkowski, Antonio Cau 
 *
 * Tempura is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Tempura is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Tempura.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/* 
 * General-purpose routines for process synchonization.
 * Ben Moszkowski
 */

/*load "lists".

load "misc".
*/

/* Declare a signal */
define make_signal(Sig) = {
    flex_list(Sig,2)
    and |Sig|=1
    and Sig[0]=false
}.

/* Declare a signal list */
define signal_list(Sig_list,n) = {
    fixed_list(Sig_list,n) and
    forall i<n: {
	make_signal(Sig_list[i])
    } 
}.

/* Status of a signal */
define signal_status(Sig) = {
    (|Sig|=2) and (Sig[0]=Sig[1])
}.

/* Execute some subprocesses */
define exec_subprocesses(Sig_list,n) = {
    signal_list(Sig_list,n)
    and {
	start_subprocesses(Sig_list);
	skip;
	terminate_subprocesses(Sig_list);
	skip
    }
}.

/* Start up all subprocesses */
define start_subprocesses(Sig_list) = {
    for i<|Sig_list| do {
	halt (Sig_list[i][0]) 
    }
    and forall i<|Sig_list|: {
	box (if |Sig_list[i]|=2 then Sig_list[i][1]=empty )
    /*and box format("start.Sig_list=%t  k=%t\n",Sig_list,k) */
    }
}.

/* Terminate all subprocesses */
define terminate_subprocesses(Sig_list) = {
    for i<|Sig_list| do {
	halt (~Sig_list[i][0])
    }
    and forall i<|Sig_list|: {
	box (if |Sig_list[i]|=2 then Sig_list[i][1]=not empty)
    /* and  box format("terminate.Sig_list=%t  k=%t\n",Sig_list,k)*/}
}.

/* begin a process */
define begin_process(Sig) = {
    {
	skip and |Sig|:=2 and Sig[0]:=true
    };
    {
	halt(Sig[1]) and stable |Sig| and stable Sig[0]
    };
    {
	skip and |Sig|:=1 and stable Sig[0]
    }
}.


/* finish a process */
define finish_process(Sig) = {
    {
	skip and |Sig|:=2 and Sig[0]:=false
    };
    {
	halt(~Sig[1]) and stable |Sig| and stable Sig[0]
    };
    {
	skip and |Sig|:=1 and stable Sig[0]
    }
}.

define sum_tree_process(Sig,Tree) = process {
    {
	begin_process(Sig) and stable Tree
    };
    {
	par_sum_tree(Tree) and stable Sig
    };
    {
	finish_process(Sig) and stable Tree
    }
}.

define par_sum_tree(Tree) = {
    if is_integer(Tree) then empty
    else {
	exists Sig_list: {
	    exec_subprocesses(Sig_list,2)
	    and sum_tree_process(Sig_list[0],Tree[0])
	    and sum_tree_process(Sig_list[1],Tree[1])
	    and stable struct(Tree)
            /*and box format("par-sum.Sig_list=%t k=%t\n",Sig_list,k)*/
	};
	{
	    skip and Tree:= Tree[0]+Tree[1]
	}
    }
}.

define process_test(t) = {
    exists Tree : {
	Tree=t 
	and par_sum_tree(Tree)
        and box format("Tree=%t\n",Tree)
    }
}.

define process_test1 = 1.

    /* run */ define rprocess_test1() = process_test(process_test1).
    
define process_test2 = [1,2].

    /* run */ define rprocess_test2() = process_test(process_test2).

define process_test3 = [[1,[2,3]],[4,5]].

    /* run */ define rprocess_test3() = process_test(process_test3).

define process_test4 = [[[1,1],[1,1]],[[1,1],[1,1]]].

    /* run */ define rprocess_test4() = process_test(process_test4).

define process_test5 = [[process_test3,process_test3],process_test3].

    /* run */ define rprocess_test5() = process_test(process_test5).
