// This file is part Tempura: Interval Temporal Logic interpreter.
//  
// Copyright (C) 1998-2013  Antonio Cau 
//
// Tempura is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Tempura is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Tempura.  If not, see <http://www.gnu.org/licenses/>.
//
//

module four_bit_adder (A4, B4, SUM5, NULL_FLAG);
input         A4,
              B4;
   output     SUM5,
              NULL_FLAG;

wire    [3:0] A4,
              B4;
wire    [4:0] SUM5;
wire    [2:0] CARRY;

one_bit_adder  Bit0 (A4[0], B4[0],     1'b0, SUM5[0], CARRY[0]);
one_bit_adder  Bit1 (A4[1], B4[1], CARRY[0], SUM5[1], CARRY[1]);
one_bit_adder  Bit2 (A4[2], B4[2], CARRY[1], SUM5[2], CARRY[2]);
one_bit_adder  Bit3 (A4[3], B4[3], CARRY[2], SUM5[3], SUM5 [4]);

nor   #1 Nor_for_zeroflag
  (NULL_FLAG, SUM5[0], SUM5[1], SUM5[2], SUM5[3], SUM5[4]);
endmodule

module one_bit_adder (A, B, CARRY_IN, SUM, CARRY_OUT);
input         A,
              B,
              CARRY_IN;
output        SUM,
              CARRY_OUT;
   
assign #1  {CARRY_OUT, SUM} = A + B + CARRY_IN;
endmodule

module test;
   reg  [3:0] A4,
              B4,
              CARRY_IN;
   wire [4:0]  SUM5;
  wire  FLAG;


   initial begin
       $dumpfile("ex6.dmp");
       $dumpvars;
   end
   
   always
     begin
        
        #5 $stop;
          #10 if ((A4<=0) && (B4<=0)) 
              begin 
                 #5 $finish;
              end
            else
               $display("!PROG: assert sum:%d:%d:!", SUM5, $time);
     end
   
   
   four_bit_adder adder(A4, B4, SUM5, FLAG);
endmodule // test
